package net.peachjean.guice.command.app;

import com.google.inject.*;
import net.peachjean.guice.command.invoke.*;

import java.io.InputStream;
import java.io.PrintStream;
import java.util.*;

/**
 * A Guice Application.  Most implementations will extend this class in order to override {@link #getModules()}.
 */
public class GuiceApp {

	/**
	 * Instantiates a GuiceApp using the streams provided.  Mostly useful for testing or embedding.
	 */
	public GuiceApp() {
	}

	protected final void run(String[] args) {
		this.run(System.in, System.out, System.err, args);
	}

	protected final void run(InputStream in, PrintStream out, PrintStream err, String[] args) {
		CommandInvokerFactory.createInvoker(getModules()).invoke(new CommandInvocation(in, out, err, args));
	}

	/**
	 * Override to provide modules to the GuiceApp.  By default returns an empty set.
	 *
	 * @return
	 */
	protected Iterable<? extends Module> getModules() {
		return Collections.emptySet();
	}

	public static void main(String[] args) {
		new GuiceApp().run(args);
	}
}
