package net.peachjean.guice.command.invoke;

import com.google.common.collect.Maps;
import com.google.inject.Inject;
import com.google.inject.Provider;
import net.peachjean.guice.command.ArgIterator;
import net.peachjean.guice.command.ArgResolver;
import net.peachjean.guice.command.CommandConfigurationException;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.*;

class ArgResolverRegistry {

	private Map<Class<?>, ArgResolver> argResolvers = new HashMap<Class<?>, ArgResolver>();
	private Provider<ArgIterator> argIterator;
	private ArgResolver arrayArgResolver;
	private ArgResolver singleArgResolver;

	@Inject
	ArgResolverRegistry(Provider<ArgIterator> argIterator, Map<Class<?>, ArgResolver> argResolverMap) {
		this.argIterator = argIterator;
		this.argResolvers = Maps.newHashMap(argResolverMap);

		if (!argResolvers.containsKey(Array.class)) {
			throw new CommandConfigurationException("No arg resolver for arrays found, this indicates a bug in this module.");
		}
		if (!argResolvers.containsKey(Object.class)) {
			throw new CommandConfigurationException("No arg resolver for single objects found, this indicates a bug in this module.");
		}
		this.arrayArgResolver = argResolvers.get(Array.class);
		this.singleArgResolver = argResolvers.get(Object.class);
	}

	public ArgResolver lookupResolver(Type type) {
		ArgResolver argResolver;
		if (type instanceof Class && ((Class) type).isArray()) {
			argResolver = arrayArgResolver;
		} else if (argResolvers.containsKey(resolveRawType(type))) {
			argResolver = argResolvers.get(resolveRawType(type));
		} else {
			argResolver = singleArgResolver;
		}
		return argResolver;
	}

	private Class<?> resolveRawType(Type type) {
		if (type instanceof Class) {
			return (Class<?>) type;
		} else if (type instanceof ParameterizedType) {
			final Type rawType = ((ParameterizedType) type).getRawType();
			return rawType instanceof Class ? (Class) rawType : null;
		} else {
			return null;
		}
	}

	public Object resolveArgValue(Method method, int parameterIndex) {
		final Type type = method.getGenericParameterTypes()[parameterIndex];
		final ArgResolver argResolver = this.lookupResolver(type);

		return argResolver.resolve(argIterator.get(), type);
	}
}
