package net.peachjean.guice.command.invoke;

import com.google.inject.Key;
import com.google.inject.OutOfScopeException;
import com.google.inject.Provider;
import com.google.inject.Scope;

class CommandScope implements Scope {

	private ThreadLocal<GuiceAppContext> appContextThreadLocal = new ThreadLocal<GuiceAppContext>();

	@Override
	public <T> Provider<T> scope(final Key<T> key, final Provider<T> unscoped) {
		return new Provider<T>() {

			@Override
			public T get() {
				final GuiceAppContext guiceAppContext = appContextThreadLocal.get();
				if (guiceAppContext == null) {
					throw new OutOfScopeException("No GuiceAppContext defined for the current thread.");
				}
				if (!guiceAppContext.scopedObjects.containsKey(key)) {
					guiceAppContext.scopedObjects.putIfAbsent(key, unscoped.get());
				}
				return getScopedObjects(guiceAppContext);
			}

			@SuppressWarnings({"unchecked"})
			private T getScopedObjects(GuiceAppContext guiceAppContext) {
				return (T) guiceAppContext.scopedObjects.get(key);
			}
		};
	}

	public void beginCommand(GuiceAppContext guiceAppContext) {
		this.appContextThreadLocal.set(guiceAppContext);
		guiceAppContext.scopedObjects.put(Key.get(GuiceAppContext.class), guiceAppContext);
	}

	public void endCommand() {
		this.appContextThreadLocal.remove();
	}

	public void performInScope(GuiceAppContext guiceAppContext, Runnable runnable) {
		try {
			this.beginCommand(guiceAppContext);
			runnable.run();
		} finally {
			this.endCommand();
		}
	}
}
