package net.peachjean.guice.command.invoke;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Provides;
import net.peachjean.guice.command.*;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;

import java.io.InputStream;
import java.io.PrintStream;

class InvokeModule extends AbstractModule {
	private CommandRegistry commandRegistry;

	public InvokeModule(CommandRegistry commandRegistry) {
		this.commandRegistry = commandRegistry;
	}

	@Override
	protected void configure() {

		CommandScope commandScope = new CommandScope();

		bind(CommandScope.class).toInstance(commandScope);
		bindScope(CommandScoped.class, commandScope);

		bind(CommandRegistry.class).toInstance(commandRegistry);
		bind(CommandInvoker.class).to(CommandInvokerImpl.class);
		bind(ArgIterator.class).to(ArgIteratorImpl.class);

		bind(CommandLineParser.class).to(GnuParser.class);

		for (OptionKey optionKey : commandRegistry.getAllOptions()) {
			Options.bindAsOption(bind(optionKey.type), optionKey.name).toProvider(new OptionProvider(optionKey, getProvider(CommandParser.class)));
		}
	}

	@Provides
	@CommandScoped
	public GuiceAppContext provideGuiceAppContext() {
		throw new IllegalStateException("This method should never be called.  There is something wrong with the library.");
	}

	@Provides
	@In
	@CommandScoped
	public InputStream provideInStream(GuiceAppContext guiceAppContext) {
		return guiceAppContext.in;
	}

	@Provides
	@Out
	@CommandScoped
	public PrintStream provideOutStream(GuiceAppContext guiceAppContext) {
		return guiceAppContext.out;
	}

	@Provides
	@Err
	@CommandScoped
	public PrintStream provideErrStream(GuiceAppContext guiceAppContext) {
		return guiceAppContext.err;
	}

	@Provides
	public OptionInfoRegistry provideOptionInfoRegistry(final Injector injector) {
		return new OptionInfoRegistry() {
			@Override
			public OptionInfo lookup(OptionKey optionKey) {
				return Options.lookupInfo(injector, optionKey.name);
			}
		};
	}
}
