package net.peachjean.guice.command.invoke.args;

import net.peachjean.guice.command.ArgIterator;
import net.peachjean.guice.command.ArgResolver;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;

public abstract class AbstractCollectionArgResolver<T extends Collection<Object>> implements ArgResolver<T> {
	@Override
	public T resolve(ArgIterator argIterator, Type type) {
		final Type componentType = resolveComponentType(type);
		T collection = createCollection();
		while (argIterator.hasMoreArgs()) {
			collection.add(argIterator.next(componentType));
		}
		return collection;
	}

	protected abstract T createCollection();

	private Type resolveComponentType(Type type) {
		if (type instanceof ParameterizedType) {
			return ((ParameterizedType) type).getActualTypeArguments()[0];
		} else {
			return String.class;
		}
	}
}
